package chamo

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Chamo.Gui_rcSource

Handling the configuration file for windows.

Sourceval rc_file : string

The user's configuration file.

Sourceval group : unit -> [ `Open ] Ocf.group

Ocf option group.

Sourceval minibuffer_history_size : int Ocf.conf_option

The option for the size of histories in minibuffers.

The "abort" key binding. This is the binding use to abort the current key press sequence. Default is C-g.

Sourceval window_key_bindings : (Stk.Key.keystate list * string) list Ocf.conf_option

The key bindings for the chamo windows. These bindings are always accessible whatever the view which has the focus.

Sourceval read : unit -> unit

Read the configuration file.

Sourceval write : unit -> unit

Write the configuration file.